@@ -30,12 +30,8 @@ ScriptInterp(), audio()
 	msgport = d->getMsgport();
 	logevents = logtrace = NULL;
 	state.logstate = (Handler)NULL;
-	state.pid = 0;
-#ifdef	HAVE_LIBEXEC
-	state.pfd = -1;
-	state.libaudio = new libaudio_t;
-	state.libaudio->line.args = state.libaudio->list;
-#endif
+	state.pfd = PFD_INVALID;
+	state.libaudio = NULL;
 	span = s;
 	timeslot = ts;
 	timeslots[ts] = this;
@@ -172,17 +168,61 @@ timeout_t BayonneSession::getJoinTimer(void)
 	return 0;
 }
 
+#ifdef	WIN32
+void BayonneSession::libWrite(const char *str)
+{
+	DWORD len = strlen(str);
+	DWORD count;
+
+	if(state.pfd == PFD_INVALID)
+		return;
+
+	if(!WriteFile(state.pfd, str, len, &count, NULL))
+	{
+		CloseHandle(state.pfd);
+		state.pfd = PFD_INVALID;
+	}
+}
+
+void BayonneSession::libClose(const char *str)
+{
+	libWrite(str);
+	if(state.pfd != PFD_INVALID)
+	{
+		CloseHandle(state.pfd);
+		state.pfd = PFD_INVALID;
+	}		
+}
+
+#else
+void BayonneSession::libWrite(const char *str)
+{
+	size_t len = strlen(str);
+	if(state.pfd != PFD_INVALID)
+	{
+		if(::write(state.pfd, (char *)str, len) < len)
+		{
+			::close(state.pfd);
+			state.pfd = PFD_INVALID;
+		}
+	}
+}
+
+void BayonneSession::libClose(const char *str)
+{
+	libWrite(str);
+	if(state.pfd != PFD_INVALID)
+	{
+		::close(state.pfd);
+		state.pfd = PFD_INVALID;
+	}
+}
+#endif
+
 uint32 BayonneSession::newTid(void)
 {
-#ifdef	HAVE_LIBEXEC
 	if(state.pid)
-	{
-		Process::cancel(state.pid);
-		if(state.pfd > -1)
-			::close(state.pfd);
-		state.pfd = -1;
-	}
-#endif
+		libClose("901 TERMINATE\n\n");
 
 	state.pid = 0;
 	snprintf(var_tid, sizeof(var_tid), "%04d+%08lx", timeslot, ++tseq);
@@ -538,7 +578,43 @@ timeout_t BayonneSession::getMSecTimeout(const char *id)
 	return atol(cp);
 }
 
-#ifdef	HAVE_LIBEXEC
+timeout_t BayonneSession::getLibexecTimeout(void)
+{
+	timeout_t timeout;
+
+	const char *cp = getMember();
+	if(cp && isdigit(*cp))
+		return atol(cp) * 1000l;
+
+	timeout = getTimeoutKeyword("timeout");
+	if(timeout == TIMEOUT_INF || !timeout)
+	{
+		cp = getMember();
+		if(cp)
+			return atol(cp) * 1000;
+	}
+	if(!timeout)
+		timeout = TIMEOUT_INF;
+	return timeout;
+}
+
+bool BayonneSession::isLibexec(const char *tsid)
+{     
+        if(state.handler != &BayonneSession::stateLibexec)    
+                goto error;
+
+        if(!state.pid)
+                goto error;
+
+        if(stricmp(var_tid, tsid))
+                goto error;
+
+        return true;
+error:
+        slog.error("libexec transaction id %s invalid", tsid);
+        return false;
+}  
+
 bool BayonneSession::setLibreset(result_t result)
 {
 	if(!state.pid)
@@ -559,19 +635,6 @@ bool BayonneSession::setLibexec(result_t result)
 	return true;
 }
 
-#else
-
-bool BayonneSession::setLibreset(result_t result)
-{
-        return false;
-} 
-
-bool BayonneSession::setLibexec(result_t result)
-{
-	return false;
-}
-#endif
-
 void BayonneSession::setRunning(void)
 {
 	setState(STATE_RUNNING);
