@@ -35,104 +35,34 @@ bool BayonneSession::stateInitial(Event *event)
 	return false;
 }
 
-#ifdef	HAVE_LIBEXEC
-
 bool BayonneSession::stateLibexec(Event *event)
 {
 	char buf[256];
 	const char *cp;
-	char *up, *p;
-	unsigned opt;
 	char evtmsg[65];
-	char bts[8];
-	unsigned count = 0;
 
 	switch(event->id)
 	{
 	case PROMPT_LIBEXEC:
-		if(!state.pid)
-			return false;
-		if(stricmp(var_tid, event->libprompt.tid))
-			return false;
-		if(state.pfd < 0)
-			return true;
-		if(*dtmf_digits)
-		{
-			state.result = RESULT_PENDING;
-			goto start;
-		}
-		memset(&state.audio, 0, sizeof(state.audio));
-		audio.offset = NULL;
-		state.audio.exitkey = true;
-		state.audio.mode = Audio::modeReadAny;
-		cp = getAudio();
-		if(cp)
-		{
-			slog.error("%s: %s", logname, cp);
-			if(signalScript(SIGNAL_FAIL))
-			{
-				setRunning();
-				return true;
-			}
-			state.result = RESULT_INVALID;
-			goto start;
-		}
-		if(event->libprompt.voice)
-		{
-			cp = event->libprompt.voice;
-			snprintf(bts, sizeof(bts), "%s", cp);
-			p = strchr(bts, '/');
-			if(p)
-				*p = 0;
-			audio.translator = BayonneTranslator::get(bts);
-			if(!audio.translator)
-			{
-				state.result = RESULT_INVALID;
-				goto start;
-			}
-			cp = audio.getVoicelib(cp);
-			if(!cp)
-			{
-				state.result = RESULT_INVALID;
-				goto start;
-			}
-			audio.voicelib = cp;
-		}
-		snprintf(state.libaudio->text, MAX_LIBINPUT, "%s", event->libprompt.text);
-		cp = strtok_r(state.libaudio->text, " \t\r\n", &p);
-		while(cp && count < (MAX_LIBINPUT / 2 - 1))
-		{
-			state.libaudio->list[count++] = cp;
-			cp = strtok_r(NULL, " \t\r\n", &p);
-		}
-		state.libaudio->list[count] = NULL;
-		state.libaudio->line.argc = count;
-		state.libaudio->line.args = state.libaudio->list;
-		state.libaudio->line.cmd = "prompt";
-		cp = audio.translator->speak(this, &state.libaudio->line);
-		if(cp)
-		{
-			slog.error("%s: %s", logname, cp);
-			state.result = RESULT_INVALID;
-			goto start;
-		}
+	case REPLAY_LIBEXEC:
 		setState(STATE_PLAY);
 		return true;				
-	case GET_LIBEXEC:
-		if(!state.pid)
-                        return false;
-                if(stricmp(var_tid, event->libinput.tid))
-                        return false;
-                if(state.pfd < 0)
-                        return true;
-		memset(&state.input, 0, sizeof(state.input));
-		state.timeout = state.input.interdigit = event->libinput.timeout;
-		state.input.count = state.input.required = event->libinput.count;
-		state.input.exit = event->libinput.exit;
+	case RECORD_LIBEXEC:
+		setState(STATE_RECORD);
+		return true;				
+	case WAIT_LIBEXEC:
+		setState(STATE_WAITKEY);
+		return true;
+	case READ_LIBEXEC:
 		setState(STATE_INPUT);
 		return true;
 	case ENTER_STATE:
-start:
+		if(!state.libaudio)
+		{
+			state.libaudio = new libaudio_t;
+			state.libaudio->line.args = state.libaudio->list; 
+		}
+	case RESTART_LIBEXEC:
 		if(state.lib && getLine() != state.lib)
 		{
 			newTid();
@@ -140,10 +70,9 @@ start:
 			return true;
 		}
 		clrAudio();
-		if(state.pid && state.pfd > -1)
+		if(state.pid && state.pfd != PFD_INVALID)
 		{
-			snprintf(buf, sizeof(buf), "100 TRANSACTION\n");
-			write(state.pfd, buf, strlen(buf));
+			libWrite("100 TRANSACTION\n");
 
 			cp = audio.var_position;
 			while(*cp && (ispunct(*cp) || *cp == '0'))
@@ -151,206 +80,53 @@ start:
 
 			if(*cp)
 			{
-				snprintf(buf, sizeof(buf), "POSITION:%s\n", audio.var_position);
-				write(state.pfd, buf, strlen(buf));
+				snprintf(buf, sizeof(buf), "POSITION: %s\n", audio.var_position);
+				libWrite(buf);
 			}
 	
 			if(state.result == RESULT_SUCCESS && *dtmf_digits)
 			{
 				snprintf(buf, sizeof(buf), "DIGITS: %s\n", dtmf_digits);
-				write(state.pfd, buf, strlen(buf));
+				libWrite(buf);
 				*dtmf_digits = 0;
 				digit_count = 0;
 			}
 
 			snprintf(buf, sizeof(buf), "RESULT: %d\n\n", state.result);
-			::write(state.pfd, buf, strlen(buf));
+			libWrite(buf);
 			state.result = RESULT_SUCCESS;
 		}
 		strcpy(audio.var_position, "00:00:00.000");
 timer:
-		cp = getMember();
-		if(cp)
-			state.timeout = atoi(cp) * 1000;
-		else
-			state.timeout = 60000;
-		startTimer(state.timeout);
+		state.timeout = getLibexecTimeout();
+		if(state.timeout != TIMEOUT_INF)
+			startTimer(state.timeout);
 		return true;
 	case ENTER_LIBEXEC:
 		if(stricmp(var_tid, event->libexec.tid))
 			return false;
 		state.result = RESULT_SUCCESS;
 		state.pid = event->libexec.pid;
+#ifdef	WIN32
+		state.pfd = event->libexec.pfd;
+#else
 		state.pfd = ::open(event->libexec.fname, O_RDWR);
 		remove(event->libexec.fname);
+#endif
 		goto timer;
 	case TIMER_EXPIRED:
-		if(state.pid)
-			kill(state.pid, SIGALRM);
-		if(state.pfd > -1)
-		{
-			::close(state.pfd);
-			state.pfd = -1;
-		}
+		libClose("902 TIMEOUT\n\n");
 		state.pid = 0;
 		newTid();
 		break;
         case DROP_LIBEXEC:
-                if(!state.pid)
-                        return false;
-                if(stricmp(var_tid, event->varexec.tid))
-                        return false;
-                if(state.pfd > -1)
-			::close(state.pfd);
-		state.pid = 0;
-		if(!signalScript(SIGNAL_EXIT))
-			error("libexec-hangup");
-		return true;		
-	case STAT_LIBEXEC:
-                if(!state.pid)
-                        return false;
-                if(stricmp(var_tid, event->varexec.tid))
-                        return false;
-                if(state.pfd < 0)
-                        return true;
-		snprintf(buf, sizeof(buf), "100 TRANSACTION\n");
-		write(state.pfd, buf, strlen(buf));
-		snprintf(buf, sizeof(buf), "STATUS: ");
-		write(state.pfd, buf, strlen(buf));
-		write(state.pfd, status, ts_count);
-		snprintf(buf, sizeof(buf), "\nRESULT: 0\n\n");
-		write(state.pfd, buf, strlen(buf));
-		return true;
-	case VAR_LIBEXEC:
-                if(!state.pid)
-                        return false;
-                if(stricmp(var_tid, event->varexec.tid))
-                        return false;
-                if(state.pfd < 0)
-                        return true;
-                snprintf(buf, sizeof(buf), "400 QUERY\n");
-                write(state.pfd, buf, strlen(buf));  
-		while(NULL != (up = strchr(event->varexec.name, '_')))
-			*up = '.';
-		if(event->varexec.value && event->varexec.size < 0)
-			catSymbol(event->varexec.name, event->varexec.value);
-		else if(event->varexec.value)
-			setSymbol(event->varexec.name, event->varexec.value, event->varexec.size);
-		cp = getSymbol(event->varexec.name);
-		snprintf(buf, sizeof(buf), "%s: ", event->varexec.name);
-		up = buf;
-		while(*up)
-		{
-			*up = toupper(*up);
-			if(*up == '.')
-				*up = '_';
-			++up;
-		}
-		write(state.pfd, buf, strlen(buf));
-		if(cp)
-		{
-			state.result = RESULT_SUCCESS;
-			write(state.pfd, cp, strlen(cp));
-		}
-		else
-			state.result = RESULT_INVALID;
-		snprintf(buf, sizeof(buf), "\nRESULT: %d\n\n", 
-			state.result);
-		state.result = RESULT_SUCCESS;
-		write(state.pfd, buf, strlen(buf));
-		return true;
-	case ARGS_LIBEXEC:
-                if(!state.pid)
-                        return false;
-                if(stricmp(var_tid, event->libexec.tid))
-                        return false;
-                if(state.pfd < 0)
-                        return true;
-                snprintf(buf, sizeof(buf), "300 ARGUMENTS\n");
-		write(state.pfd, buf, strlen(buf));
-		frame[stack].index = 0;
-		cp = getOption(NULL);
-		while(NULL != (cp = getOption(NULL)))
-		{
-			snprintf(buf, sizeof(buf), "%s: ", cp + 1);
-			up = buf;
-			while(*up)
-			{
-				*up = toupper(*up);
-				if(*up == '.')
-					*up = '_';
-				++up;
-			}
-			write(state.pfd, buf, strlen(buf));
-			cp = getContent(cp);
-			if(cp && *cp)
-				write(state.pfd, cp, strlen(cp));
-			write(state.pfd, "\n", 1);
-		}
-		opt = 0;
-		while(NULL != (cp = state.lib->args[opt]))
-		{
-			if(*cp == '=')
-			{
-				snprintf(buf, sizeof(buf), "%s: ", ++cp);
-				up = buf;
-				while(*up)
-				{
-					*up = toupper(*up);
-					++up;
-				}
-				write(state.pfd, buf, strlen(buf));
-				cp = getKeyword(cp);
-				if(cp && *cp)
-					write(state.pfd, cp, strlen(cp));
-				write(state.pfd, "\n", 1);
-				opt += 2;
-			}
-			else
-				++opt;
-		}
-		write(state.pfd, "\n", 1);
-		return true;	
-	case HEAD_LIBEXEC:
 		if(!state.pid)
 			return false;
 		if(stricmp(var_tid, event->libexec.tid))
 			return false;
-		if(state.pfd < 0)
-			return true;
-		snprintf(buf, sizeof(buf), "200 HEADER\n");
-		write(state.pfd, buf, strlen(buf));
-		snprintf(buf, sizeof(buf), "SESSION: %s\n", var_sid);
-		write(state.pfd, buf, strlen(buf));
-		if(*dtmf_digits)
-		{
-			snprintf(buf, sizeof(buf), "DIGITS: %s\n", dtmf_digits);
-			write(state.pfd, buf, strlen(buf));
-			*dtmf_digits = 0;
-			digit_count = 0;
-		}
-		cp = getSymbol("session.caller");
-		if(cp)
-		{
-			snprintf(buf, sizeof(buf), "CALLER: %s\n", cp);
-			write(state.pfd, buf, strlen(buf));
-		}
-		cp = getSymbol("session.dialed");
-		if(cp)
-		{
-			snprintf(buf, sizeof(buf), "DIALED: %s\n", cp);
-			write(state.pfd, buf, strlen(buf));
-		}
-		cp = voicelib;
-		if(!cp)
-			cp = "none/prompts";
-		snprintf(buf, sizeof(buf), "VOICE: %s\n", cp);
-		write(state.pfd, buf, strlen(buf));
-		snprintf(buf, sizeof(buf), "START: %s %s\n", var_date, var_time);
-		write(state.pfd, buf, strlen(buf));
-		snprintf(buf, sizeof(buf), "IFACE: %s\n", getExternal("session.interface"));
-		write(state.pfd, buf, strlen(buf));
-		write(state.pfd, "\n", 1);
+		newTid();
+		if(!signalScript(SIGNAL_EXIT))
+			error("libexec-hangup");
 		return true;		
 	case EXIT_LIBEXEC:
 		if(!state.pid)
@@ -399,20 +175,6 @@ bool BayonneSession::stateLibreset(Event *event)
 	return enterCommon(event);
 }
 
-#else
-
-bool BayonneSession::stateLibexec(Event *event)
-{
-	return false;
-}
-
-bool BayonneSession::stateLibreset(Event *event)
-{
-	return false;
-}
-
-#endif
-
 bool BayonneSession::stateReset(Event *event)
 {
 	if(enterReset(event))
@@ -502,6 +264,38 @@ bool BayonneSession::stateStart(Event *event)
 	return enterCommon(event);
 }
 
+bool BayonneSession::stateWaitkey(Event *event)
+{
+	switch(event->id)
+	{
+	case DTMF_KEYUP:
+		digit_count = 1;
+		dtmf_digits[0] = getChar(event->dtmf.digit);
+		dtmf_digits[1] = 0;
+gotkey:
+		if(setLibexec(RESULT_PENDING))
+			return true;
+		advance();
+		setRunning();
+		return true;
+	case ENTER_STATE:
+		if(*dtmf_digits)
+			goto gotkey;
+		if(state.timeout)
+		{
+			startTimer(state.timeout);
+			return true;
+		}
+	case TIMER_EXPIRED:
+		if(setLibexec(RESULT_TIMEOUT))
+			return true;
+		advance();
+		setRunning();
+		return true;
+	}
+	return enterCommon(event);
+}
+
 bool BayonneSession::stateSleep(Event *event)
 {
 	switch(event->id)
@@ -947,6 +741,9 @@ bool BayonneSession::stateTone(Event *event)
 			setState(STATE_HANGUP);
 			return true;
 		}
+		if(setLibreset(RESULT_COMPLETE))
+			return true;
+
 		advance();
 		setRunning();
 		return true;
@@ -1095,6 +892,7 @@ exitkey:
 	case AUDIO_IDLE:
 		if(setLibreset(RESULT_COMPLETE))
 			return true;
+
 		advance();
 		setRunning();
 		return true;
@@ -1183,6 +981,9 @@ exitkey:
 		}
 		break;
 	case AUDIO_IDLE:
+		if(setLibreset(RESULT_COMPLETE))
+			return true;
+
 		advance();
 		setRunning();
 		return true;
@@ -1835,6 +1636,7 @@ bool BayonneSession::filterPosting(Event *event)
 
 bool BayonneSession::enterCommon(Event *event)
 {
+	result_t result;
 	char evtmsg[65];
 
 	switch(event->id)
@@ -2000,6 +1802,14 @@ stop:
 			return true;
 		}
 
+		if(event->errmsg)
+			result = RESULT_FAILED;
+		else
+			result = RESULT_COMPLETE;
+
+		if(setLibreset(result))
+			return true;
+
 		if(!updated)
 		{
 			if(event->errmsg)
@@ -2047,6 +1857,13 @@ childerr:
 	case TIMER_EXPIRED:
 		if(!image)
 			return false;
+
+		if(thread)
+		{
+			if(setLibreset(RESULT_TIMEOUT))
+				return true;
+		}
+
 		if(!signalScript(SIGNAL_TIMEOUT))
 			error("timer-expired");
 		setRunning();
