@@ -394,28 +394,28 @@ sub result($$) {
 
 sub get($$) {
 	my($self, $buf) = @_;
-	$self->command("get $value");
+	$self->command("get $buf");
 	return $self->{query};
 }
 
 # set symbol value
 
-sub set($$) {
-	my($self, $value) = @_;
-	return $self->command("set $value");
+sub set($$$) {
+	my($self, $id, $value) = @_;
+	return $self->command("set $id $value");
 }
 
-sub add($$) {
-        my($self, $value) = @_;
-        return $self->command("add $value");
+sub add($$$) {
+        my($self, $id, $value) = @_;
+        return $self->command("add $id $value");
 } 
 
 # size a symbol
 
-sub size($$) {
-	my($self, $buf) = @_;
+sub size($$$) {
+	my($self, $id, $buf) = @_;
 	my($size) = $buf - 0;
-	return $self->command("new $size");
+	return $self->command("new $id $size");
 }
 	
 # build prompt
