@@ -348,6 +348,13 @@ const char *Checks::chkLibexec(Line *line, ScriptImage *img)
 			return "libexec arguments must be variables";
 	}
 
+	if(!useKeywords(line, "=token=results=prefix=encoding=framing=voice=extension=timeout"))
+		return "invalid keyword used in libexec";
+
+	cp = findKeyword(line, "results");
+	if(cp && *cp != '&')
+		return "results must be saved into &var reference";
+
 	return NULL;
 }
 
