@@ -616,9 +616,22 @@ bool Methods::scrPosition(void)
 
 bool Methods::scrLibexec(void)
 {
-	const char *cp = getMember();
+	const char *cp = getKeyword("results");
+	Symbol *sym = NULL;
 
-	if(!Libexec::create(this))
+	if(cp)
+		sym = mapSymbol(cp);
+
+	if(cp && !sym)
+	{
+		error("libexec-no-results");
+		return true;
+	}
+
+	if(sym)
+		clear(sym);
+
+	if(!BayonneSysexec::create(this))
 	{
 		error("libexec-failed");
 		return true;
@@ -989,6 +1002,18 @@ bool Methods::scrSync(void)
 	return false;
 }	
 
+bool Methods::scrWaitkey(void)
+{
+	state.timeout = getTimeoutValue();
+        if(!state.timeout || *dtmf_digits)
+        {
+                advance();
+                return true;
+        } 
+	setState(STATE_WAITKEY);
+	return false;
+}
+
 bool Methods::scrSleep(void)
 {
 	state.timeout = getTimeoutValue();
