@@ -20,6 +20,10 @@
 #include "private.h"
 #endif
 
+#ifdef	HAVE_LIBEXEC
+#include "libexec.h"
+#endif
+
 #ifndef	VERSION
 #define	VERSION "0.3.1"
 #endif
@@ -192,6 +196,7 @@ public:
 	bool scrEcho(void);
 	bool scrPickup(void);
 	bool scrHangup(void);
+	bool scrWaitkey(void);
 	bool scrSleep(void);
 	bool scrSync(void);
 	bool scrCleardigits(void);
@@ -274,20 +279,20 @@ public:
 
 #ifdef	HAVE_LIBEXEC
 
-class Libexec : protected Thread, protected Bayonne
+class Sysexec : protected Thread, protected Bayonne
 {
 private:
 	static bool exiting;
 	static int iopair[2];
-	static Libexec *libexec;
+	static Sysexec *libexec;
 
 	static void execute(unsigned slot, const char *cmd);
 	static void readline(char *buf, unsigned max);
 
 	void run(void);
 
-	Libexec();
-	~Libexec();
+	Sysexec();
+	~Sysexec();
 
 public:
 	static bool create(BayonneSession *s);
