@@ -40,6 +40,9 @@ $TGI->print("COMMAND RESULT $result\n");
 $TGI->print("LAST COMMAND RESULT BEFORE EXIT $TGI->{result}\n");
 $TGI->result("CODE $result");
 
+$file = $TGI->filename("tmp:edit");
+$TGI->print("FILE NAME $file\n");
+
 exit
 
 
